@@ -1,9 +1,21 @@
-"""Class to handle the ground truth of a video"""
+"""Class to handle the ground truth of a video
+
+HumanDetections are loaded in with a video name that is completely
+independent of the path. Users can set the fmdt variables WATEC6_DIR
+and WATEC12_DIR to execute database tests
+"""
 
 import pandas as pd
 import math
 import fmdt.core
+import fmdt.utils
+import fmdt.args
 from fmdt.core import TrackedObject
+import numpy as np
+import os
+
+WATEC6_DIR:  str = "./"
+WATEC12_DIR: str = "./"
 
 class HumanDetection:
 
@@ -32,6 +44,21 @@ class HumanDetection:
         return (
             f"""({self.video_name}, f0: {self.start_frame}, fT: {self.end_frame}, pos0: ({self.start_x}, {self.start_y}), posT: ({self.end_x}, {self.end_y}))"""
         )
+
+    def __lt__(self, other):
+        return self.video_name < other.video_name
+    
+    def is_draco_6(self) -> bool:
+        """Return true if part of the Draconids6mm series"""
+        # vid_name = self.video_name.split("/")[-1]
+        v, _ = fmdt.utils.decompose_video_filename(self.video_name)
+        return "Draconids-6mm" in v 
+    
+    def is_draco_12(self) -> bool:
+        """Return true if part of the Draconids6mm series"""
+        # vid_name = self.video_name.split("/")[-1]
+        v, _ = fmdt.utils.decompose_video_filename(self.video_name)
+        return "Draconids-12mm" in v 
     
     def delta_x(self) -> float:
         """Displacement in the x_axis direction"""
@@ -51,6 +78,8 @@ class HumanDetection:
         return self.end_frame - self.start_frame
 
     def slope(self) -> float:
+        if self.delta_x() == 0:
+            return float("inf")
         return self.delta_y() / self.delta_x()
     
     def direction(self) -> float:
@@ -66,7 +95,7 @@ class HumanDetection:
         yi = self.start_y + (self.dx() * self.slope()) * f_prime
         return (xi, yi)
 
-    def is_detected(self, tracking_list: list[TrackedObject]) -> bool:
+    def is_detected_in_list(self, tracking_list: list[TrackedObject]) -> bool:
 
         # keep only meteors
         # detected_m = [m for m in tracking_list if m.is_meteor()]
@@ -75,20 +104,36 @@ class HumanDetection:
             if are_objects_the_same(self, tracked):
                 return True
 
-        return False 
+        return False
     
-    # def __repr__(self) -> str:
-        # return self.__str__()
+    def test_detection_vary_light(
+            self,
+            vid: str,
+            offset: int = 25,
+            light_min_start: int = 150,
+            light_min_end: int = 230,
+            k_trials: int = 10,
+            log: bool = False
+        ):
+
+        return vary_light_intervals(
+            vid=vid,
+            truth=self,
+            offset=offset,
+            light_min_start=light_min_start,
+            light_min_end=light_min_end,
+            k_trials=k_trials,
+            log=log
+        )
 
     @staticmethod
-    def init_ground_truth(database_filename: str):
-        HumanDetection.GROUND_TRUTH = read_human_detection_csv(database_filename) 
-        return read_human_detection_csv(database_filename)
+    def init_ground_truth(database_filename: str, video_db_dir: str = "./"):
+        HumanDetection.GROUND_TRUTH = read_human_detection_csv(database_filename, video_db_dir) 
+        return read_human_detection_csv(database_filename, video_db_dir)
+
+
 
-    # I want to convert a csv of truth values into a HumanDetection object
 
-    # Ultimately, I will need a function that takes in a csv of truth values and converts it into 
-    # a list of HumanDetection objects
 
 # Take a row in a csv file from https://docs.google.com/spreadsheets/d/1yYsMy0nnLAsqzTYOoEjof00wVu6oP91vCiLS7orxMpg/edit#gid=365172110
 # and convert it to a HumanDetection object
@@ -104,13 +149,13 @@ def csv_to_human(csv_row: str) -> HumanDetection:
 
 # Take a single row of a pandas data frame of the csv data and return
 # a python HumanDetection object
-def df_row_to_human(row: pd.Series) -> HumanDetection:
+def df_row_to_human(row: pd.Series, dir_prepend: str = "") -> HumanDetection:
 
     # This function is performing NOOOO sanity checks
     # like do any of these fields even actually exist 
     # in our data frame??
 
-    return HumanDetection(row["video_name"],
+    return HumanDetection(dir_prepend + row["video_name"],
                           int(row["start_frame"]),
                           int(row["end_frame"]),
                           float(row["start_x"]),
@@ -118,12 +163,12 @@ def df_row_to_human(row: pd.Series) -> HumanDetection:
                           float(row["end_x"]),
                           float(row["end_y"]))
 
-def read_human_detection_csv(csv_filename: str) -> list[HumanDetection]:
+def read_human_detection_csv(csv_filename: str, db_dir: str = "") -> list[HumanDetection]:
     df = pd.read_csv(csv_filename)
-    return [df_row_to_human(df.iloc[i]) for i in range(len(df.index))]
+    return [df_row_to_human(df.iloc[i], db_dir) for i in range(len(df.index))]
 
-def init_ground_truth(database_filename: str) -> list[HumanDetection]:
-    return HumanDetection.init_ground_truth()
+def init_ground_truth(database_filename: str, vid_db_dir: str = "./") -> list[HumanDetection]:
+    return HumanDetection.init_ground_truth(database_filename, video_db_dir=vid_db_dir)
 
 # GROUND_TRUTH : list[HumanDetection] = None
 
@@ -144,7 +189,7 @@ def is_meteor_detected(meteor: HumanDetection, tracking_list: list[TrackedObject
 
     return False 
 
-def are_objects_the_same(meteor: HumanDetection, tracked_obj: TrackedObject) -> bool:
+def are_objects_the_same(meteor: HumanDetection, tracked_obj: TrackedObject, log: bool = False) -> bool:
     """Check if a HumanDetected meteor is the same as a tracked obj
     
     A tracked object is a dictionary with keys:
@@ -157,14 +202,10 @@ def are_objects_the_same(meteor: HumanDetection, tracked_obj: TrackedObject) ->
     """
 
     # Are the lifetimes the same???
-    # TODO
     # ====================== Lifetime ===========================
     lifetime_meteor = (meteor.start_frame, meteor.end_frame)
     lifetime_object = (tracked_obj.start_frame, tracked_obj.end_frame)
 
-    nframes_meteor = lifetime_meteor[1] - lifetime_meteor[0]
-    nframes_object = lifetime_object[1] - lifetime_object[0]
-
     # meteor:                        [***********************]           
     # object:        [------]
     if lifetime_object[1] < lifetime_meteor[0]:
@@ -175,19 +216,19 @@ def are_objects_the_same(meteor: HumanDetection, tracked_obj: TrackedObject) ->
     if lifetime_object[0] > lifetime_meteor[1]:
         return False
     
-    # TODO compare the ratio of slopes rather than the difference
     # =============== Compare Direction ==========================
     angle_meteor_rad = meteor.direction() 
     angle_object_rad = tracked_obj.direction() 
-    # We should really be comparing their angles...
 
-    MAX_ANGLE_DIFF = 0.5 # ARBITRARY AS FUCK
+    MAX_ANGLE_DIFF = 0.5 #!!! ARBITRARY
 
-    print(f"Meteor has angle: {angle_meteor_rad}")
-    print(f"Object has angle: {angle_object_rad}")
+    if log:
+        print(f"Meteor has angle: {angle_meteor_rad}")
+        print(f"Object has angle: {angle_object_rad}")
 
     if abs(angle_meteor_rad - angle_object_rad) > MAX_ANGLE_DIFF:
-        print("Angles are too far apart")
+        if log:
+            print("Angles are too far apart")
         return False
     
 
@@ -243,8 +284,6 @@ def are_objects_the_same(meteor: HumanDetection, tracked_obj: TrackedObject) ->
 
         dist = math.sqrt(sqr(pos_object[0] - pos_meteor[0]) + sqr(pos_object[1] - pos_meteor[1]))
 
-        # print(dist)
-
         if dist > MAX_DIST:
             # print(f"Position at frame {f} is too far apart (dist: {dist})")
             return False
@@ -252,9 +291,63 @@ def are_objects_the_same(meteor: HumanDetection, tracked_obj: TrackedObject) ->
     return True
         
 
-# I want to compare the human results with the automatic results.
+def vary_light_intervals(
+        vid: str,
+        truth: HumanDetection,
+        offset: int,
+        light_min_start: int,
+        light_min_end: int,
+        k_trials: int,
+        log: bool = False
+    ) -> list[fmdt.args.Args]:
+    """Vary light min and light max to try and detect the truth
+    
+    Return a list of args that succesfully detected this truth
+    """
 
-# First thing I need is a pipeline to get the automatic results
+    args = []
+
+    for lmin in np.linspace(light_min_start, light_min_end, k_trials, dtype=int):
+
+
+        light_min = int(lmin)
+        light_max = light_min + offset
+
+        track_file = "tracks.txt"
+        track_bb_path = "track_bb.txt"
+
+        vname, ext = fmdt.utils.decompose_video_filename(vid)
+        vname = f"{vname}_off{offset}"
+
+        res = fmdt.detect(vid, 
+                        trk_out_path=track_file,
+                        log=True,
+                        light_min=light_min,
+                        light_max=light_max,
+                        trk_all=True,
+                        trk_bb_path=track_bb_path)
+
+        if len(res.tracking_list) != 0:
+
+            if not os.path.isdir(vname):
+                os.mkdir(vname)
+
+            if truth.is_detected_in_list(res.tracking_list):
+                addon = "_detected"
+                if truth.is_detected_in_list([t for t in res.tracking_list if t.is_meteor()]):
+                    addon = f"{addon}_as_meteor"
+                    args.append(res)
+            else:
+                addon = ""
+
+            fmdt.visu(vid, track_file, track_bb_path, f"{vname}/lmin{light_min}_lmax{light_max}{addon}.{ext}", show_id=True)
+
+        if log: 
+            meteors = [m for m in res.tracking_list if m.is_meteor()]
+            for m in meteors:
+                print(m)
+
+    return args
 
 def main() -> None:
     # dets = read_human_detection_csv("human_detection.csv")
